/// Base data model with factory method to parse a json response from an API call
abstract class BaseModel{
  BaseModel();

  factory BaseModel.fromJson(Map<String,dynamic> json){
    throw UnimplementedError();
  }
}

/// Data model about the page results from any paginated API
class PageData<T>{

  /// The list of data returned inside this page
  List<T> data;

  /// The status code results for this page
  int count;

  /// Error message in case of error
  String next;

  /// Has a next page?
  String previous;

  PageData({
    required this.data,
    required this.count,
    required this.next,
    required this.previous
  });

  factory PageData.fromJson(Map<String, dynamic> json){
    return PageData(
      data: List<T>.generate(json['data'].length, ((element) => BaseModel.fromJson(json['data'][element]) as T)),
      count: json['count'],
      next: json['next'],
      previous: json['previous']
    );
  }

}